var express = require('express'),
    httpAgent = require('http-agent'),
    jsdom = require('jsdom');

// create an instance of HTTPServer
var app = express.createServer();

app.configure( function() {
	// disable layout
	app.set("view options", {layout: false});

	// make a custom html template
	app.register('.html', {
		compile: function(str, options){
			return function(locals){
				str.replace(/\t/g, '&nbsp;&nbsp;&nbsp;&nbsp;');
				return str;
			};
        }
	});
});

// define a route to the root of the site
app.get('/', function(req, res) {
	res.render( __dirname + '/index.html' );
});

// define a route to handle the request for /scrape
app.get('/scrape', function(req, res) {
	// create a variable for the query param in the path
	var date = req.param('date');

	// pass a host, path and query string to visit
	var agent = httpAgent.create('www.imdb.com', ['boxoffice/?region=us&date=' + date]);

	// init a string for the eventual response
	var theResponseHTML = '';

	// add the listener for the event that will fire after the host is visited
	agent.addListener('next', function (err, agent) {
	  	// set up the environment for the DOM
		// pass the markup from the host as the body
		// load scripts
		jsdom.env({
		    html: agent.body,
		    scripts: [
		      'http://code.jquery.com/jquery-1.5.min.js'
		    ]
		}, function (err, window) {
	    	// expose jQuery to the $ variable
			var $ = window.jQuery;

			// begin creating the response
			theResponseHTML += 'Top Ten: ' + date;

			// use jQuery to parse the host markup
			for ( i=1; i <= 10; i++ ) {
				var theRank = $('#main table tr:nth-child('+i+') td:nth-child(1)').text(),
			        theTitleAndYear = $('#main table tr:nth-child('+i+') td:nth-child(3)').text(),
			        theWeekend = $('#main table tr:nth-child('+i+') td:nth-child(4)').text(),
			        theGross = $('#main table tr:nth-child('+i+') td:nth-child(5)').text(),
			        theWeeks = $('#main table tr:nth-child('+i+') td:nth-child(6)').text();

				theRank = $.trim( theRank );
			    theTitleAndYear = $.trim( theTitleAndYear );
			    theWeekend = $.trim( theWeekend );
			    theGross = $.trim( theGross );
			    theWeeks = $.trim( theWeeks );

				theResponseHTML += '<ul>' +
						'<li>the rank: ' + theRank + '</li>' +
						'<li>the title and year: ' + theTitleAndYear + '</li>' +
						'<li>the weekend: ' + theWeekend + '</li>' +
						'<li>the gross: ' + theGross + '</li>' +
						'<li>the weeks: ' + theWeeks + '</li>' +
					'</ul>';
			}

			// fire next
			agent.next();

			// send the response to the client
			res.send(theResponseHTML);
		});
	});

	// provide logging in the terminal
	agent.addListener('stop', function (agent) {
	  	console.log('the agent has stopped');
	});

	// start the agent
	agent.start();
});

// the server listens on 3000
app.listen(3000);